package org.eclipse.jdt.internal.compiler.apt.dispatch;

package .tmp;

import java.lang.annotation.Annotation;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import javax.annotation.processing.RoundEnvironment;
import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.TypeElement;
import javax.lang.model.util.ElementFilter;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.apt.dispatch.AnnotationDiscoveryVisitor;
import org.eclipse.jdt.internal.compiler.apt.dispatch.BaseProcessingEnvImpl;
import org.eclipse.jdt.internal.compiler.apt.model.Factory;
import org.eclipse.jdt.internal.compiler.apt.model.TypeElementImpl;
import org.eclipse.jdt.internal.compiler.apt.util.ManyToMany;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;

public class RoundEnvImpl implements RoundEnvironment {
  private final BaseProcessingEnvImpl _processingEnv;
  
  private final boolean _isLastRound;
  
  private final CompilationUnitDeclaration[] _units;
  
  private final ManyToMany<TypeElement, Element> _annoToUnit;
  
  private final ReferenceBinding[] _binaryTypes;
  
  private final Factory _factory;
  
  private Set<Element> _rootElements = null;
  
  public RoundEnvImpl(CompilationUnitDeclaration[] units, ReferenceBinding[] binaryTypeBindings, boolean isLastRound, BaseProcessingEnvImpl env) {
    this._processingEnv = env;
    this._isLastRound = isLastRound;
    this._units = units;
    this._factory = this._processingEnv.getFactory();
    AnnotationDiscoveryVisitor visitor = new AnnotationDiscoveryVisitor(this._processingEnv);
    if (this._units != null) {
      byte b;
      int i;
      CompilationUnitDeclaration[] arrayOfCompilationUnitDeclaration;
      for (i = (arrayOfCompilationUnitDeclaration = this._units).length, b = 0; b < i; ) {
        CompilationUnitDeclaration unit = arrayOfCompilationUnitDeclaration[b];
        unit.scope.suppressImportErrors = true;
        unit.traverse((ASTVisitor)visitor, unit.scope);
        unit.scope.suppressImportErrors = false;
        b++;
      } 
    } 
    this._annoToUnit = visitor._annoToElement;
    if (binaryTypeBindings != null)
      collectAnnotations(binaryTypeBindings); 
    this._binaryTypes = binaryTypeBindings;
  }
  
  private void collectAnnotations(ReferenceBinding[] referenceBindings) {
    byte b;
    int i;
    ReferenceBinding[] arrayOfReferenceBinding;
    for (i = (arrayOfReferenceBinding = referenceBindings).length, b = 0; b < i; ) {
      ReferenceBinding referenceBinding = arrayOfReferenceBinding[b];
      if (referenceBinding instanceof ParameterizedTypeBinding)
        referenceBinding = ((ParameterizedTypeBinding)referenceBinding).genericType(); 
      AnnotationBinding[] annotationBindings = Factory.getPackedAnnotationBindings(referenceBinding.getAnnotations());
      byte b1;
      int j;
      AnnotationBinding[] arrayOfAnnotationBinding1;
      for (j = (arrayOfAnnotationBinding1 = annotationBindings).length, b1 = 0; b1 < j; ) {
        AnnotationBinding annotationBinding = arrayOfAnnotationBinding1[b1];
        TypeElement anno = (TypeElement)this._factory.newElement((Binding)annotationBinding.getAnnotationType());
        Element element = this._factory.newElement((Binding)referenceBinding);
        this._annoToUnit.put(anno, element);
        b1++;
      } 
      FieldBinding[] fieldBindings = referenceBinding.fields();
      int k;
      FieldBinding[] arrayOfFieldBinding1;
      for (k = (arrayOfFieldBinding1 = fieldBindings).length, j = 0; j < k; ) {
        FieldBinding fieldBinding = arrayOfFieldBinding1[j];
        annotationBindings = Factory.getPackedAnnotationBindings(fieldBinding.getAnnotations());
        byte b2;
        int n;
        AnnotationBinding[] arrayOfAnnotationBinding;
        for (n = (arrayOfAnnotationBinding = annotationBindings).length, b2 = 0; b2 < n; ) {
          AnnotationBinding annotationBinding = arrayOfAnnotationBinding[b2];
          TypeElement anno = (TypeElement)this._factory.newElement((Binding)annotationBinding.getAnnotationType());
          Element element = this._factory.newElement((Binding)fieldBinding);
          this._annoToUnit.put(anno, element);
          b2++;
        } 
        j++;
      } 
      MethodBinding[] methodBindings = referenceBinding.methods();
      MethodBinding[] arrayOfMethodBinding1;
      for (int m = (arrayOfMethodBinding1 = methodBindings).length; k < m; ) {
        MethodBinding methodBinding = arrayOfMethodBinding1[k];
        annotationBindings = Factory.getPackedAnnotationBindings(methodBinding.getAnnotations());
        byte b2;
        int n;
        AnnotationBinding[] arrayOfAnnotationBinding;
        for (n = (arrayOfAnnotationBinding = annotationBindings).length, b2 = 0; b2 < n; ) {
          AnnotationBinding annotationBinding = arrayOfAnnotationBinding[b2];
          TypeElement anno = (TypeElement)this._factory.newElement((Binding)annotationBinding.getAnnotationType());
          Element element = this._factory.newElement((Binding)methodBinding);
          this._annoToUnit.put(anno, element);
          b2++;
        } 
        k++;
      } 
      ReferenceBinding[] memberTypes = referenceBinding.memberTypes();
      collectAnnotations(memberTypes);
      b++;
    } 
  }
  
  public Set<TypeElement> getRootAnnotations() {
    return Collections.unmodifiableSet(this._annoToUnit.getKeySet());
  }
  
  public boolean errorRaised() {
    return this._processingEnv.errorRaised();
  }
  
  public Set<? extends Element> getElementsAnnotatedWith(TypeElement a) {
    if (a.getKind() != ElementKind.ANNOTATION_TYPE)
      throw new IllegalArgumentException("Argument must represent an annotation type"); 
    Binding annoBinding = ((TypeElementImpl)a)._binding;
    if (0L != (annoBinding.getAnnotationTagBits() & 0x1000000000000L)) {
      Set<Element> annotatedElements = new HashSet<>(this._annoToUnit.getValues(a));
      ReferenceBinding annoTypeBinding = (ReferenceBinding)annoBinding;
      for (TypeElement element : ElementFilter.typesIn(getRootElements())) {
        ReferenceBinding typeBinding = (ReferenceBinding)((TypeElementImpl)element)._binding;
        addAnnotatedElements(annoTypeBinding, typeBinding, annotatedElements);
      } 
      return Collections.unmodifiableSet(annotatedElements);
    } 
    return Collections.unmodifiableSet(this._annoToUnit.getValues(a));
  }
  
  private void addAnnotatedElements(ReferenceBinding anno, ReferenceBinding type, Set<Element> result) {
    if (type.isClass() && 
      inheritsAnno(type, anno))
      result.add(this._factory.newElement((Binding)type)); 
    byte b;
    int i;
    ReferenceBinding[] arrayOfReferenceBinding;
    for (i = (arrayOfReferenceBinding = type.memberTypes()).length, b = 0; b < i; ) {
      ReferenceBinding element = arrayOfReferenceBinding[b];
      addAnnotatedElements(anno, element, result);
      b++;
    } 
  }
  
  private boolean inheritsAnno(ReferenceBinding element, ReferenceBinding anno) {
    ReferenceBinding searchedElement = element;
    while (true) {
      if (searchedElement instanceof ParameterizedTypeBinding)
        searchedElement = ((ParameterizedTypeBinding)searchedElement).genericType(); 
      AnnotationBinding[] annos = Factory.getPackedAnnotationBindings(searchedElement.getAnnotations());
      byte b;
      int i;
      AnnotationBinding[] arrayOfAnnotationBinding1;
      for (i = (arrayOfAnnotationBinding1 = annos).length, b = 0; b < i; ) {
        AnnotationBinding annoBinding = arrayOfAnnotationBinding1[b];
        if (annoBinding.getAnnotationType() == anno)
          return true; 
        b++;
      } 
      if ((searchedElement = searchedElement.superclass()) == null)
        return false; 
    } 
  }
  
  public Set<? extends Element> getElementsAnnotatedWith(Class<? extends Annotation> a) {
    String canonicalName = a.getCanonicalName();
    if (canonicalName == null)
      throw new IllegalArgumentException("Argument must represent an annotation type"); 
    TypeElement annoType = this._processingEnv.getElementUtils().getTypeElement(canonicalName);
    if (annoType == null)
      return Collections.emptySet(); 
    return getElementsAnnotatedWith(annoType);
  }
  
  public Set<? extends Element> getRootElements() {
    if (this._units == null)
      return Collections.emptySet(); 
    if (this._rootElements == null) {
      Set<Element> elements = new HashSet<>(this._units.length);
      byte b;
      int i;
      CompilationUnitDeclaration[] arrayOfCompilationUnitDeclaration;
      for (i = (arrayOfCompilationUnitDeclaration = this._units).length, b = 0; b < i; ) {
        CompilationUnitDeclaration unit = arrayOfCompilationUnitDeclaration[b];
        if (unit.scope != null && unit.scope.topLevelTypes != null) {
          byte b1;
          int j;
          SourceTypeBinding[] arrayOfSourceTypeBinding;
          for (j = (arrayOfSourceTypeBinding = unit.scope.topLevelTypes).length, b1 = 0; b1 < j; ) {
            SourceTypeBinding binding = arrayOfSourceTypeBinding[b1];
            Element element = this._factory.newElement((Binding)binding);
            if (element == null)
              throw new IllegalArgumentException("Top-level type binding could not be converted to element: " + binding); 
            elements.add(element);
            b1++;
          } 
        } 
        b++;
      } 
      if (this._binaryTypes != null) {
        ReferenceBinding[] arrayOfReferenceBinding;
        for (i = (arrayOfReferenceBinding = this._binaryTypes).length, b = 0; b < i; ) {
          ReferenceBinding typeBinding = arrayOfReferenceBinding[b];
          Element element = this._factory.newElement((Binding)typeBinding);
          if (element == null)
            throw new IllegalArgumentException("Top-level type binding could not be converted to element: " + typeBinding); 
          elements.add(element);
          b++;
        } 
      } 
      this._rootElements = elements;
    } 
    return this._rootElements;
  }
  
  public boolean processingOver() {
    return this._isLastRound;
  }
}
